#![forbid(unsafe_code)]

use std::{fs::create_dir_all, net::SocketAddr, process::Stdio};

use web::state::{SharedState, TemplateState};
use executor::{executors, executors_id, Executor};

mod web;
pub mod executor;

fn check() {
    let res = std::process::Command::new("python")
        .arg("--version")
        .stdout(Stdio::null())
        .stderr(Stdio::null())
        .spawn();
    if res.is_err() {
        panic!("Python is not installed. Make sure that `python` executable exists in $PATH and is Python 3");
    }
    
    if create_dir_all("/tmp/sandy-tmp").is_err() {
        panic!("Cannot create temporary directory on /tmp/sandy-tmp");
    }

    let execs = executors();
    let available = execs
        .iter()
            .map(|x| {
                if ! x.available() {
                    log::warn!("{} is not available in this environment!", x.id())
                }
                x
            })
            .filter(|x| x.available())
            .collect::<Vec<&Box<dyn Executor>>>()
        .len();

    if available == 0 {
        panic!("No executors are available.");
    }
}

#[tokio::main]
async fn main() {

    if cfg!(debug_assertions) {
        femme::with_level(log::LevelFilter::Debug)
    } else {
        femme::with_level(log::LevelFilter::Info)
    }

    check();
    
    let state = SharedState {
        template: TemplateState {
            langs: executors_id()
        }
    };
    let routes = web::routes(state);

    println!("Running on 0.0.0.0:80");

    warp::serve(routes).run("0.0.0.0:80".parse::<SocketAddr>().unwrap()).await;
}
